package java.awt;

import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import java.util.HashSet;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.awt.peer.TextAreaPeer;
import java.util.Set;

public class TextArea extends TextComponent
{
    int rows;
    int columns;
    private static final String base = "text";
    private static int nameCounter;
    public static final int SCROLLBARS_BOTH = 0;
    public static final int SCROLLBARS_VERTICAL_ONLY = 1;
    public static final int SCROLLBARS_HORIZONTAL_ONLY = 2;
    public static final int SCROLLBARS_NONE = 3;
    private int scrollbarVisibility;
    private static Set<AWTKeyStroke> forwardTraversalKeys;
    private static Set<AWTKeyStroke> backwardTraversalKeys;
    private static final long serialVersionUID = 3692302836626095722L;
    private int textAreaSerializedDataVersion;
    
    private static native void initIDs();
    
    public TextArea() throws HeadlessException {
        this("", 0, 0, 0);
    }
    
    public TextArea(String s) throws HeadlessException {
        this(s, 0, 0, 0);
    }
    
    public TextArea(int n, int n2) throws HeadlessException {
        this("", n, n2, 0);
    }
    
    public TextArea(String s, int n, int n2) throws HeadlessException {
        this(s, n, n2, 0);
    }
    
    public TextArea(String s, int n, int n2, int scrollbarVisibility) throws HeadlessException {
        super(s);
        this.textAreaSerializedDataVersion = 2;
        this.rows = ((n >= 0) ? n : 0);
        this.columns = ((n2 >= 0) ? n2 : 0);
        if (scrollbarVisibility >= 0 && scrollbarVisibility <= 3) {
            this.scrollbarVisibility = scrollbarVisibility;
        }
        else {
            this.scrollbarVisibility = 0;
        }
        this.setFocusTraversalKeys(0, TextArea.forwardTraversalKeys);
        this.setFocusTraversalKeys(1, TextArea.backwardTraversalKeys);
    }
    
    @Override
    String constructComponentName() {
        synchronized (TextArea.class) {
            return "text" + TextArea.nameCounter++;
        }
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            if (this.peer == null) {
                this.peer = this.getToolkit().createTextArea(this);
            }
            super.addNotify();
        }
    }
    
    public void insert(String s, int n) {
        this.insertText(s, n);
    }
    
    @Deprecated
    public synchronized void insertText(String str, int n) {
        TextAreaPeer textAreaPeer = (TextAreaPeer)this.peer;
        if (textAreaPeer != null) {
            textAreaPeer.insert(str, n);
        }
        this.text = this.text.substring(0, n) + str + this.text.substring(n);
    }
    
    public void append(String s) {
        this.appendText(s);
    }
    
    @Deprecated
    public synchronized void appendText(String s) {
        this.insertText(s, this.getText().length());
    }
    
    public void replaceRange(String s, int n, int n2) {
        this.replaceText(s, n, n2);
    }
    
    @Deprecated
    public synchronized void replaceText(String str, int endIndex, int beginIndex) {
        TextAreaPeer textAreaPeer = (TextAreaPeer)this.peer;
        if (textAreaPeer != null) {
            textAreaPeer.replaceRange(str, endIndex, beginIndex);
        }
        this.text = this.text.substring(0, endIndex) + str + this.text.substring(beginIndex);
    }
    
    public int getRows() {
        return this.rows;
    }
    
    public void setRows(int rows) {
        int rows2 = this.rows;
        if (rows < 0) {
            throw new IllegalArgumentException("rows less than zero.");
        }
        if (rows != rows2) {
            this.rows = rows;
            this.invalidate();
        }
        return;
    }
    
    public int getColumns() {
        return this.columns;
    }
    
    public void setColumns(int columns) {
        int columns2 = this.columns;
        if (columns < 0) {
            throw new IllegalArgumentException("columns less than zero.");
        }
        if (columns != columns2) {
            this.columns = columns;
            this.invalidate();
        }
        return;
    }
    
    public int getScrollbarVisibility() {
        return this.scrollbarVisibility;
    }
    
    public Dimension getPreferredSize(int n, int n2) {
        return this.preferredSize(n, n2);
    }
    
    @Deprecated
    public Dimension preferredSize(int n, int n2) {
        synchronized (this.getTreeLock()) {
            TextAreaPeer textAreaPeer = (TextAreaPeer)this.peer;
            return (textAreaPeer != null) ? textAreaPeer.getPreferredSize(n, n2) : super.preferredSize();
        }
    }
    
    @Override
    public Dimension getPreferredSize() {
        return this.preferredSize();
    }
    
    @Deprecated
    @Override
    public Dimension preferredSize() {
        synchronized (this.getTreeLock()) {
            return (this.rows > 0 && this.columns > 0) ? this.preferredSize(this.rows, this.columns) : super.preferredSize();
        }
    }
    
    public Dimension getMinimumSize(int n, int n2) {
        return this.minimumSize(n, n2);
    }
    
    @Deprecated
    public Dimension minimumSize(int n, int n2) {
        synchronized (this.getTreeLock()) {
            TextAreaPeer textAreaPeer = (TextAreaPeer)this.peer;
            return (textAreaPeer != null) ? textAreaPeer.getMinimumSize(n, n2) : super.minimumSize();
        }
    }
    
    @Override
    public Dimension getMinimumSize() {
        return this.minimumSize();
    }
    
    @Deprecated
    @Override
    public Dimension minimumSize() {
        synchronized (this.getTreeLock()) {
            return (this.rows > 0 && this.columns > 0) ? this.minimumSize(this.rows, this.columns) : super.minimumSize();
        }
    }
    
    @Override
    protected String paramString() {
        String str;
        switch (this.scrollbarVisibility) {
            case 0: {
                str = "both";
                break;
            }
            case 1: {
                str = "vertical-only";
                break;
            }
            case 2: {
                str = "horizontal-only";
                break;
            }
            case 3: {
                str = "none";
                break;
            }
            default: {
                str = "invalid display policy";
                break;
            }
        }
        return super.paramString() + ",rows=" + this.rows + ",columns=" + this.columns + ",scrollbarVisibility=" + str;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        objectInputStream.defaultReadObject();
        if (this.columns < 0) {
            this.columns = 0;
        }
        if (this.rows < 0) {
            this.rows = 0;
        }
        if (this.scrollbarVisibility < 0 || this.scrollbarVisibility > 3) {
            this.scrollbarVisibility = 0;
        }
        if (this.textAreaSerializedDataVersion < 2) {
            this.setFocusTraversalKeys(0, TextArea.forwardTraversalKeys);
            this.setFocusTraversalKeys(1, TextArea.backwardTraversalKeys);
        }
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new TextArea.AccessibleAWTTextArea();
        }
        return this.accessibleContext;
    }
    
    static {
        TextArea.nameCounter = 0;
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        TextArea.forwardTraversalKeys = KeyboardFocusManager.initFocusTraversalKeysSet("ctrl TAB", new HashSet());
        TextArea.backwardTraversalKeys = KeyboardFocusManager.initFocusTraversalKeysSet("ctrl shift TAB", new HashSet());
    }
    
    protected class AccessibleAWTTextArea extends TextComponent.AccessibleAWTTextComponent
    {
        private static final long serialVersionUID = 3472827823632144419L;
        
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            accessibleStateSet.add(AccessibleState.MULTI_LINE);
            return accessibleStateSet;
        }
    }
}
